/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Manuel Meier
 *   Co-authors:
 *      Cyrill Burgener
 *
 */

/**
    @file OrxoKart.cc
    @brief Implementation of the OrxoKart class. Sets up the whole Minigame
*/

#include "OrxoKart.h"
#include "OrxoKartFlag.h"
#include "OrxoKartKart.h"
#include "OrxoKartOrigin.h"
#include "OrxoKartTile.h"
#include "core/CoreIncludes.h"

namespace orxonox
{
    RegisterUnloadableClass(OrxoKart);

    OrxoKart::OrxoKart(Context* context) : Gametype(context)
    {
        RegisterObject(OrxoKart);

        this->origin_ = nullptr;
        this->numberOfFlags_ = 1;
        this->firstTick_ = true;

        this->setHUDTemplate("OrxoKartHUD");
    }

    void OrxoKart::tick(float dt)
    { 
        SUPER(OrxoKart, tick, dt);

        if(this->firstTick_ && this->origin_)
        {
            this->firstTick_ = false;
            this->wayPoints = std::vector<OrxoKartTile*>(3, nullptr);

            int n = this->origin_->getNumCells();
            int s = this->origin_->getCellSize();
            int map_1[] =   {1,1,1,1,1,1,1,1,0,0,0
                            ,1,0,0,0,0,0,0,1,1,1,0
                            ,1,0,1,1,1,1,0,0,0,1,1
                            ,1,1,1,0,0,1,1,0,0,0,1
                            ,0,0,0,0,0,0,1,0,1,1,1
                            ,1,1,1,1,1,1,1,0,1,0,0
                            ,1,0,0,0,0,0,0,0,1,0,0
                            ,1,0,0,0,0,0,0,0,1,0,0
                            ,1,1,1,1,1,0,0,0,1,1,1
                            ,0,0,0,0,1,0,0,0,0,0,1
                            ,0,0,0,0,1,1,1,2,1,1,1};

            int map_2[] =  {1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,
                            1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,
                            1,0,1,1,1,1,1,1,1,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,
                            1,0,1,0,0,0,0,0,0,0,1,0,1,0,1,1,1,1,1,1,1,1,1,0,0,
                            1,0,1,0,1,1,1,1,1,0,1,0,1,0,1,0,0,0,0,0,0,0,1,0,0,
                            1,0,1,0,1,0,0,0,1,0,1,0,1,0,1,0,1,1,1,1,1,0,1,0,0,
                            1,0,1,0,1,1,1,0,1,0,1,0,1,0,1,0,1,0,0,0,1,0,1,0,0,
                            1,0,1,0,0,0,1,0,1,0,1,0,1,0,1,0,1,0,1,1,1,0,1,0,0,
                            1,0,1,1,1,1,1,0,1,0,1,0,1,0,1,0,1,0,1,0,0,0,1,0,0,
                            1,0,0,0,0,0,0,0,1,0,1,0,1,0,1,0,1,0,1,1,1,1,1,0,0,
                            1,1,1,1,1,1,1,1,1,0,1,0,1,0,1,0,1,0,0,0,0,0,0,0,0,
                            0,0,0,0,0,0,0,0,0,0,1,0,1,0,1,0,1,1,1,1,1,1,1,1,1,
                            0,1,1,1,1,1,1,1,1,1,1,0,1,0,1,0,0,0,0,0,0,0,0,0,1,
                            0,1,0,0,0,0,0,0,0,0,0,0,1,0,1,1,1,1,0,0,0,0,0,0,1,
                            0,1,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,1,0,0,0,0,0,0,1,
                            0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,1,1,1,1,1,
                            0,1,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,
                            0,1,0,1,1,1,0,0,1,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,
                            0,1,0,1,0,1,0,0,1,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,
                            0,1,0,1,0,1,0,0,1,1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,
                            0,1,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
                            0,1,0,1,0,1,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,1,
                            0,1,1,1,0,1,0,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,1,
                            0,0,0,0,0,1,1,1,0,0,0,1,0,0,0,0,0,0,0,0,1,1,1,0,1,
                            0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,2,1,1,1,0,1,1,1};



            int *levelcodeArray =  map_1;

            // int z = 0;
            //Generate floor according to levelcode
            for(int i=0; i<n; i++){
                for(int j=0; j<n; j++){
                    if (int type = levelcodeArray[i*n + j]) {
                        OrxoKartTile* tile = new OrxoKartTile(origin_->getContext());
                        tile->init((n-1-i)*s, j*s, s, type);
                        // tiles.push_back(tile);
                        //++z;
                        if ( i == 10 && j == 4 )
                            wayPoints.at(0) = tile;

                        if ( i == 0 && j == 0) 
                            wayPoints.at(1) = tile;
                        if ( i == 10 && j == 7)
                            wayPoints.at(2) = tile;
                    }
                }   
            }
            //bottom flag
            
            OrxoKartFlag* flag = new OrxoKartFlag(origin_->getContext());
            flag->init(n, s);
            flag_ = flag;
            


            

        } //firsttick end
        /*
        for (std::vector<OrxoKartTile*>::iterator it = tiles.begin(); it != tiles.end(); ++it ) {
            if ((*it)->getCollided())
                tiles.erase(it);
        }
        if (tiles.empty())
            numberOfFlags_ = 0;
        */
        if (wayPointCounter >= 3)
            numberOfFlags_ = 0;
        else if (wayPointCounter < 3 && wayPointCounter >= 0 && wayPoints.at(wayPointCounter) != nullptr) {
            if (wayPoints.at(wayPointCounter)->getCollided()) {
                for (int i = 0; i < 3; ++i) {
                    wayPoints.at(i)->setCollided(false);
                }
                ++wayPointCounter;
            }
        }
        else {
            orxout() << " Should not happen, look in OrxoKart.cc";
        }





        // Check if ship collided with the flag
        if(flag_->getCollided()){
            OrxoKartKart* player = flag_->getObjectofCollision();
            player->setPosition(Vector3(80*0, 20, 80*8));
            player->setOrientation(Quaternion(1, 0, 0, 0));   // 0, 0, 0
            player->setVelocity(Vector3(0,0,0));
            player->setAcceleration(Vector3(0,0,0));
            flag_->setCollided(false); 
            wayPointCounter = 0;


            // flag_->destroyLater();
           // flags_.erase (flags_.begin()+i);
            //numberOfFlags_ = 0;
        }
        
    }
}
